#!/usr/bin/env python
# coding: utf-8

# Lehenengo, datuak entrenamendu eta testatze datuetan banatu dira.
# 

# In[ ]:


import pandas as pd
from sklearn.model_selection import train_test_split

# Datu-basea irakurri
dataset = pd.read_csv('F:/GrAL/4bert/Prueba.csv')

# datuen %10 testatze-datu eta %90 entrenamendu-datu bezala banatu, 'CLASE' aldagaia proportzionalki banatuz
train_data, test_data = train_test_split(dataset, test_size=0.1, random_state=13, stratify=dataset['CLASE'])

# Bi datu-baseak csv fitxategietan gorde
train_data.to_csv('F:/GrAL/4bert/dftrain.csv', index=False)
test_data.to_csv('F:/GrAL/4bert/dftest.csv', index=False)


# Behin entrenamendu- eta testatze-datuak lortu direla, neurona-sarea programatu da.

# In[ ]:


import pandas as pd
import numpy as np
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras import layers
from tensorflow.keras.callbacks import EarlyStopping
from sklearn.metrics import confusion_matrix, f1_score

# Datu-baseak irakurri
dftrain = pd.read_csv('F:/GrAL/4bert/dftrain.csv')
dftest = pd.read_csv('F:/GrAL/4bert/dftest.csv')

# Entrenamendu datuak egokitu: embedding-ak 'str' bezala gordeta zaude eta np.array-ak izan behar dira. Klaseak 1-etik 4-ra izan beharrean 0-tik 3-ra izan behar dira.
X_str = dftrain['EMBEDDING']
y_mal = dftrain['CLASE']

X = []
y = []

for emb_str in X_str:
    embedding = []
    emb_str = emb_str[1:-1]
    emb = emb_str.split(', ')
    for num in emb:
        embedding.append(float(num))
    X.append(embedding)

for clase in y_mal:
    y.append(int(clase-1))

X_train = np.asarray(X)
y_train = np.asarray(y)

#Testatze datuak ere modu berean egokitu
X_str = dftest['EMBEDDING']
y_mal = dftest['CLASE']

X = []
y = []

for emb_str in X_str:
    embedding = []
    emb_str = emb_str[1:-1]
    emb = emb_str.split(', ')
    for num in emb:
        embedding.append(float(num))
    X.append(embedding)

for clase in y_mal:
    y.append(int(clase-1))

X_test = np.asarray(X)
y_test = np.asarray(y)


# Ausazko hazia zehaztu eredua exekutatzen den bakoitzean berdin mantentzeko
np.random.seed(13)
tf.random.set_seed(13)

# Entrenamenduan eredua hobetzeaz uzten badu, entrenamendua geldituko da
early_stopping = EarlyStopping(monitor='val_loss', patience=2)

# Neurona-sarearen egitura
model = tf.keras.Sequential([tf.keras.layers.Dense(5, activation='relu', input_shape=(768,)), tf.keras.layers.Dense(4, activation='softmax')])

# Optimizazio algoritmoa zehaztu
adam = tf.keras.optimizers.Adam(
    learning_rate=0.00095,
    beta_1=0.99,
    beta_2=0.999,
    epsilon=1e-07)

# Neurona-sarea konpilatu
model.compile(loss='sparse_categorical_crossentropy', optimizer=adam, metrics=['accuracy'], callbacks=[early_stopping])

# Neurona-sarea entrenatu
entreno = model.fit(X_train, y_train, epochs=25, batch_size=32, validation_data=(X_test, y_test))


# In[ ]:


# Testatze-datuekin ebaluatu
loss, accuracy = model.evaluate(X_test, y_test, verbose=False)
y_ebal = np.argmax(model.predict(X_test), axis=-1)

# Konfusio matrizea kalkulatu
km = confusion_matrix(y_test, y_ebal)
print(km)

# F1 score kalkulatu
f1 = f1_score(y_test, y_ebal, average='weighted')
print("F1 score:", f1)


# Konfusio matrizea irudikatzeko, hurrengo kodea egin da.

# In[ ]:


import seaborn as sns
import matplotlib.pyplot as plt

# Konfusio matrizea
km = [[0, 1, 3, 0], [0, 35, 44, 4], [0, 16, 185, 50], [0, 7, 95, 103]]

# Kolorea aukertau
color = sns.color_palette("Blues")

# Konfusio matrizearen itxura zehaztu
ax = sns.heatmap(km, annot=True, cmap=color, fmt='d', cbar=False, square=True, annot_kws={"ha": 'center', "va": 'center'})

# Koloreen intentsitate-barra sortu
sns.heatmap(km, annot=True, cmap=color, fmt='d')

# Ardatzak zehaztu
plt.xticks([0.5, 1.5, 2.5, 3.5], range(1, 5))
plt.yticks([0.5, 1.5, 2.5, 3.5], range(1, 5))
plt.xlabel("Igarritako balioak")
plt.ylabel("Balio errealak")

# Irudia sortu
plt.show()


# Azkenik, neurona sarea hobetzeko, entrenamenduko datu-basean lagin gutxi zituzten klaseetan laginak erreplikatu dira, klase guztiak lagin-kopuru bera izan dezaten. 
# 

# In[ ]:


import pandas as pd
from sklearn.model_selection import train_test_split

# Datu-basea irakurri
df = pd.read_csv('F:/GrAL/4bert/dftrain.csv')

# Datuak klaseetan banatu
dfC1 = df[df['CLASE']==1]
dfC2 = df[df['CLASE']==2]
dfC3 = df[df['CLASE']==3]
dfC4 = df[df['CLASE']==4]

# Lagin kopuru handiena hartu
m = max([len(dfC1), len(dfC2), len(dfC3), len(dfC4)])

# Klase bakoitzeko datuak erreplikatu m lagin lortu arte
df1 = dfC1.sample(m, replace=True)
df2 = dfC2.sample(m, replace=True)
df3 = dfC3.sample(m, replace=True)
df4 = dfC4.sample(m, replace=True)

# Datu-base batean elkartu
dataset = pd.concat([df1,df2,df3,df4], ignore_index=True)

# Datu-base berria csv fitxategi batean gorde
dataset.to_csv('F:/GrAL/4bert/dftrain2.csv', index=False)


# Neurona-sare berriaren kodea aurrekoarena bezalakoa da.
